#!/usr/bin/perl

=head1 NAME

rsync-here.pl


=head1 SYNOPSIS

Wrapper around rsync program to provide an easy calling syntax for rsyncing against a predefined target.


=head1 DESCRIPTION

Before starting, your need to configure some specific variables in this file. Please see section CONFIGURATION.
rsync-here.pl expects a single argument on the command line. Give it a full directory path and 
it will run the rsync program in the following style: (abstract)

    cd /$root; \     # go to one level on top of the specified source path
    $rsync -azuv -R -e ssh --progress $source $target


=head1 CONFIGURATION

Please edit "rsync-here.pl" to configure it, there is no ".ini"-style configuration file yet.

=over 4

=item -
  $sh:    /path/to/your/sh

=item -
  $rsync: /path/to/your/rsync

=item -
  $target_prefix: username@host.mydomain.org

=item -
  $target_base: /path/to/your/backup-repository-base

=back

The hostname of your machine will be appended to "$target_base", on windows also the volumename.


=head1 PREREQUISITES

=over 4

=item *
  Perl

=item *
  Rsync

=item *
  SSH

=back

You also may find it convenient to setup ssh working with keys. (use ssh-keygen [-d])


=head1 COPYRIGHT

This program is free software. You may copy or
redistribute it under the same terms as Perl itself.


=head1 TODO

=over 4

=item
   + windows compatibility

=item
   + waiting for keypress on exit: better handling

=item
   + write output to STDOUT while working!

=item
   - ease ssh-key-setup for non-interactive use, include doc at min!

=item
   - read "user - config" variables from %ENV

=item
   - make it possible to sync single files

=back

=cut

use strict;
use warnings;
use Sys::Hostname;

# user - config
  my $sh = "sh";
  my $rsync = "rsync";
  my $target_prefix = 'joko@martha';
  my $target_base = '~/backup';
  my $bool_dry_run = 1;
  

# used variables
  my $rootpath;
  my $rootpath_append;
  my $hostname;
  my $source;
  my $source_volume;
  my $target;

# runtime - config
BEGIN { sub RUNNING_IN_HELL () { $^O eq 'MSWin32' } }

$rootpath = "/";
if (RUNNING_IN_HELL) {
  $rootpath = "/cygdrive";
}


# main - get and validate arguments
  $source = shift;
  if (!$source) {
    print "no source path given, exit.", "\n";
    exit;
  }

# main - modify source
  if (RUNNING_IN_HELL) {
    $source =~ s/^(\w)://i;
    $source_volume = $1;
    if (!$source_volume) {
      print "we are running on windows, please give full path including volume. (e.g. \"c:\\home\")";
      exit;
    }
    # convert backslashes to slashes
    $source =~ s/\\/\//g;
    $source_volume = lc $source_volume;
    #$sourcePath = "/cygdrive/$sourcePath_volume$sourcePath";
    $rootpath_append = "/$source_volume/";
  }
  # strip leading and trailing slash, if any
  $source =~ s/^\///;
  $source =~ s/\/$//;
  

# main - modify target
  $hostname = hostname;
  $hostname ||= "DUMMY";
  $target = "$target_prefix:$target_base/$hostname/";
  if (RUNNING_IN_HELL) {
    $target .= "$source_volume/";
  }


# main - build rsync command
  my $cmd;

  my $opt_dry_run = "";
  $opt_dry_run = "--dry-run" if ($bool_dry_run);

  if ($rootpath_append) { $rootpath .= $rootpath_append; }

  $cmd = "$sh -c 'cd $rootpath; $rsync -azuv -e ssh -R --progress --times $opt_dry_run \"$source\" \"$target\"'";
  print "synchronizing directories with...", "\n";
  print "cmd: ", $cmd, "\n";

# main - run rsync
  open(STDERR,'>&STDOUT');        # redirect error to out
  $| = 1;                                       # no buffering
  system($cmd);                            # execute system command
  
# wait for pressing <ENTER>
  print "\n";
  print "sleeping, press <ENTER> to continue...";
  read STDIN, (my $content), 1;
  
